test('bkprun01', normal, backpack_run, [''])
test('bkprun02', normal, backpack_run, [''])
test('bkprun03', normal, backpack_run, [''])
test('bkprun04', normal, backpack_run, [''])
test('bkprun05', exit_code(1), backpack_run, [''])
test('bkprun06', normal, backpack_run, [''])
test('bkprun07', normal, backpack_run, [''])
test('bkprun08', normal, backpack_run, [''])
test('bkprun09', normal, backpack_run, ['-O'])
test('T13955', normal, backpack_run, [''])
