test('Ppr001', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr001'])
test('Ppr002', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr002'])
test('Ppr003', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr003'])
test('Ppr004', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr004'])
test('Ppr005', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr005'])
test('Ppr006', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr006'])
test('Ppr007', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr007'])
test('Ppr008', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr008'])
test('Ppr009', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr009'])
test('Ppr010', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr010'])
test('Ppr011', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr011'])
test('Ppr012', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr012'])
test('Ppr013', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr013'])
test('Ppr014', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr014'])
test('Ppr015', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr015'])
test('Ppr016', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr016'])
test('Ppr017', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr017'])
test('Ppr018', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr018'])
test('Ppr019', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr019'])
test('Ppr020', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr020'])
test('Ppr021', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr021'])
test('Ppr022', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr022'])
test('Ppr023', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr023'])
test('Ppr024', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr024'])
test('Ppr025', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr025'])
test('Ppr026', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr026'])
test('Ppr027', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr027'])
test('Ppr028', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr028'])
test('Ppr029', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr029'])
test('Ppr030', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr030'])
test('Ppr031', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr031'])
test('Ppr032', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr032'])
test('Ppr033', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr033'])
test('Ppr034', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr034'])
test('Ppr035', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr035'])
test('Ppr036', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr036'])
test('Ppr037', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr037'])
test('Ppr038', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr038'])
test('Ppr039', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr039'])
test('Ppr040', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr040'])
test('Ppr041', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr041'])
test('Ppr042', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr042'])
test('Ppr043', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr043'])
test('Ppr044', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr044'])
test('Ppr045', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr045'])
test('Ppr046', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr046'])
test('Ppr047', expect_fail, run_command, ['$MAKE -s --no-print-directory ppr047'])
test('Ppr048', ignore_stderr, run_command, ['$MAKE -s --no-print-directory ppr048'])
test('T13199', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T13199'])
test('T13050p', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T13050p'])
test('T13550', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T13550'])
test('T13942', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T13942'])
test('T14289', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T14289'])
test('T14289b', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T14289b'])
test('T14289c', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T14289c'])
test('T14306', ignore_stderr, run_command, ['$MAKE -s --no-print-directory T14306'])
