test('bkpfail01', normal, backpack_typecheck_fail, [''])
test('bkpfail03', normal, backpack_typecheck_fail, [''])
test('bkpfail04', normal, backpack_typecheck_fail, [''])
test('bkpfail05', normal, backpack_compile_fail, [''])
test('bkpfail06', normal, backpack_compile_fail, [''])
test('bkpfail07', normal, backpack_compile_fail, [''])
test('bkpfail09', normal, backpack_compile_fail, [''])
test('bkpfail10', normal, backpack_compile_fail, [''])
test('bkpfail11', normal, backpack_compile_fail, [''])
test('bkpfail12', normal, backpack_compile_fail, [''])
test('bkpfail13', normal, backpack_compile_fail, [''])
test('bkpfail14', normal, backpack_compile_fail, [''])
test('bkpfail15', normal, backpack_compile_fail, [''])
test('bkpfail16', normal, backpack_compile_fail, [''])
test('bkpfail17', normal, backpack_compile_fail, [''])
test('bkpfail18', normal, backpack_compile_fail, [''])
test('bkpfail19', normal, backpack_compile_fail, [''])
test('bkpfail20', normal, backpack_compile_fail, [''])
test('bkpfail21', normal, backpack_compile_fail, [''])
test('bkpfail22', normal, backpack_compile_fail, [''])
test('bkpfail23', normal, backpack_compile_fail, [''])
test('bkpfail24', normal, backpack_compile_fail, [''])
test('bkpfail25', normal, backpack_compile_fail, [''])
test('bkpfail26', normal, backpack_compile_fail, [''])
test('bkpfail27', normal, backpack_compile_fail, [''])
test('bkpfail28', normal, backpack_compile_fail, [''])
test('bkpfail29', normal, backpack_compile_fail, [''])
test('bkpfail30', normal, backpack_compile_fail, [''])
test('bkpfail31', normal, backpack_compile_fail, [''])
test('bkpfail32', normal, backpack_compile_fail, [''])
test('bkpfail33', normal, backpack_compile_fail, [''])
test('bkpfail34', normal, backpack_compile_fail, [''])
test('bkpfail35', normal, backpack_compile_fail, [''])
test('bkpfail36', normal, backpack_compile_fail, [''])
test('bkpfail37', normal, backpack_compile_fail, [''])
test('bkpfail38', normal, backpack_compile_fail, [''])
test('bkpfail39', expect_broken(13068), backpack_compile_fail, [''])
test('bkpfail40', normal, backpack_compile_fail, [''])
test('bkpfail41', normal, backpack_compile_fail, [''])
test('bkpfail42', normal, backpack_compile_fail, [''])
test('bkpfail43', normal, backpack_compile_fail, [''])
test('bkpfail44', normal, backpack_compile_fail, [''])
test('bkpfail45', normal, backpack_compile_fail, [''])
test('bkpfail46', normal, backpack_compile_fail, [''])
test('bkpfail47', normal, backpack_compile_fail, [''])
test('bkpfail48', normal, backpack_compile_fail, [''])
test('bkpfail49', normal, backpack_compile_fail, [''])
