{-# LANGUAGE NoImplicitPrelude #-}
module B (fingerprintFingerprints) where

import A
import C

fingerprintFingerprints :: [Fingerprint] -> Fingerprint
fingerprintFingerprints = undefined mkTyConApp

undefined :: a
undefined = undefined

