setTestOpts(only_ways(['normal']));

test('mdorun001', normal, compile_and_run, [''])
test('mdorun002', normal, compile_and_run, [''])
test('mdorun003', normal, compile_and_run, [''])
test('mdorun004', normal, compile_and_run, [''])
test('mdorun005', normal, compile_and_run, [''])
