{-# LANGUAGE Trustworthy #-}
{-# OPTIONS_GHC -fwarn-trustworthy-safe #-}

-- | This module is marked trustworthy but should be inferable as -XSafe.
-- Warning enabled.
module TrustworthySafe02 where

g :: Int
g = 1

