
{-# LANGUAGE NegativeLiterals #-}

module T7895 where

import Data.Int
import Data.Word

v1 :: Word8
v1 = 300

v2 :: Int8
v2 = -129

v3 :: Int8
v3 = -128

v4 :: Int8
v4 = -127

v5 :: Int8
v5 = -5

v6 :: Int8
v6 = 127

