module T14128Main where

import T14128

-- Overview:
--   * T14128Main is imports T14128
--   * T14128Type defines datatype MyType
--   * T14128 has a boot file and defines a orphan Show instance; it SOURCE
--     imports T14128
--   * T14128a has a boot file which SOURCE imports T14128
