# Leave these off until the CPS pass is finished and integrated.
#
# These tests are not in their final format yet, but they provide
# a starting point.

test('cmm001', skip, compile, ['-O -fasm -ddump-cps-cmm'])
test('cmm002', skip, compile, ['-fasm -ddump-cps-cmm'])

test('cps001', skip, compile, ['-ddump-cps-cmm'])
test('cps002', skip, compile, ['-ddump-cps-cmm'])
test('cps003', skip, compile, ['-ddump-cps-cmm'])
test('cps004', skip, compile, ['-ddump-cps-cmm'])
test('cps005', skip, compile, ['-ddump-cps-cmm'])
test('cps006', skip, compile, ['-ddump-cps-cmm'])
test('cps007', skip, compile, ['-ddump-cps-cmm'])
test('cps008', skip, compile, ['-ddump-cps-cmm'])
test('cps009', skip, compile, ['-ddump-cps-cmm'])
test('cps010', skip, compile, ['-ddump-cps-cmm'])
test('cps011', skip, compile, ['-ddump-cps-cmm'])
test('cps012', skip, compile, ['-ddump-cps-cmm'])
test('cps013', skip, compile, ['-ddump-cps-cmm'])
test('cps014', skip, compile, ['-ddump-cps-cmm'])
test('cps015', skip, compile, ['-ddump-cps-cmm'])
test('cps016', skip, compile, ['-ddump-cps-cmm'])
test('cps017', skip, compile, ['-ddump-cps-cmm'])
test('cps018', skip, compile, ['-ddump-cps-cmm'])
test('cps019', skip, compile, ['-ddump-cps-cmm'])
test('cps020', skip, compile, ['-ddump-cps-cmm'])
test('cps021', skip, compile, ['-ddump-cps-cmm'])
test('cps022', skip, compile, ['-ddump-cps-cmm'])
test('cps023', skip, compile, ['-ddump-cps-cmm'])
test('cps024', skip, compile, ['-ddump-cps-cmm'])
test('cps025', skip, compile, ['-ddump-cps-cmm'])
test('cps026', skip, compile, ['-ddump-cps-cmm'])

