-- !!! checking that special ids are correctly handled.
module ShouldCompile where

as :: [as]
as = [head as]

qualified :: [qualified]
qualified = [head qualified]

hiding :: [hiding]
hiding = [head hiding]

export :: [export]
export = [head export]

label :: [label]
label = [head label]

dynamic :: [dynamic]
dynamic = [head dynamic]

unsafe :: [unsafe]
unsafe = [head unsafe]

interruptible :: [interruptible]
interruptible = [head interruptible]

stdcall :: [stdcall]
stdcall = [head stdcall]

ccall :: [ccall]
ccall = [head ccall]

