-- The contents of this file are irrelevant. It is merely
-- the target for compilation by the T7702Plugin, which
-- exhibits the space leak in Trac #7702
module Main where

main :: IO ()
main = return ()
