# Args to vtc are: extra compile flags

test('rn003', normal, compile, [''])
test('rn005', normal, compile, [''])
test('rn006', normal, compile, [''])
test('rn009', [extra_files(['Imp10Aux.hs', 'Imp10Aux.hs-boot'])], multimod_compile, ['rn009', '-v0'])
# rn10 tests the same things as 009,011,012
#test('rn010', normal, multimod_compile, ['rn010', '-v0'])
test('rn011', [extra_files(['Imp100Aux.hs', 'Imp100Aux.hs-boot'])], multimod_compile, ['rn011', '-v0'])
test('rn012', [extra_files(['Imp500Aux.hs', 'Imp500Aux.hs-boot'])], multimod_compile, ['rn012', '-v0'])
test('rn013', normal, compile, [''])
test('rn017', [extra_files(['RnAux017.hs', 'RnAux017.hs-boot'])], multimod_compile, ['rn017', '-v0'])
test('rn019', normal, compile, [''])
test('rn020', normal, compile, [''])
test('rn022', normal, compile, [''])
test('rn023', normal, compile, [''])
test('rn024', normal, compile, [''])
test('rn025', normal, compile, [''])
test('rn026', normal, compile, [''])
test('rn027', normal, compile, [''])
test('rn028', normal, compile, [''])
test('rn029', normal, compile, [''])
test('rn031', normal, compile, [''])
test('rn032', normal, compile, [''])
test('rn033', normal, compile, [''])
test('rn034', normal, compile, [''])
test('rn035', normal, compile, [''])
test('rn036', normal, compile, [''])
test('rn037', normal, compile, [''])

# Missing:
# test('rn038', normal, compile, [''])

test('rn039', normal, compile, [''])
test('rn040', normal, compile, ['-fwarn-unused-binds -fwarn-unused-matches'])
test('rn041', expect_broken_for(10181, ['optasm', 'optllvm']), compile, [''])
test('rn042', [extra_files(['Rn042_A.hs'])], multimod_compile, ['rn042', '-v0'])
test('rn043', [extra_files(['Rn043_A.hs', 'Rn043_B.hs'])], multimod_compile, ['rn043', '-v0'])
test('rn044', [extra_files(['Rn044_A.hs', 'Rn044_B.hs'])], multimod_compile, ['rn044', '-v0'])
test('rn045', normal, compile, [''])
test('rn046', normal, compile, ['-W'])
test('rn047', normal, compile, ['-W'])
test('rn048', normal, compile, ['-W'])
test('rn049', normal, compile, ['-W'])

test('rn050', [extra_files(['Rn050_A.hs'])], multimod_compile, ['rn050', '-v0'])

test('rn051', normal, compile, [''])

test('rn052', [extra_files(['Rn052Aux.hs'])], multimod_compile, ['rn052', '-v0'])

test('rn053', [extra_files(['Rn053_A.hs', 'Rn053_B.hs'])], multimod_compile, ['rn053', '-v0'])

test('rn054', normal, compile, [''])
test('rn055', normal, compile, [''])
test('rn056', normal, compile, [''])
test('rn057', normal, compile, [''])
test('rn058', normal, compile, [''])
test('rn059', [extra_files(['Rn059_A.hs', 'Rn059_B.hs'])], multimod_compile, ['rn059', '-v0'])
test('rn060', normal, compile, [''])
test('rn061', normal, compile, [''])
test('rn062', normal, compile, [''])
test('rn063', normal, compile, [''])
test('rn064', normal, compile, [''])
test('rn065', [extra_files(['Rn065A.hs'])], multimod_compile, ['rn065', '-v0'])

test('rn066', [extra_files(['Rn066_A.hs'])], multimod_compile, ['rn066', '-v0'])

test('rn067', [extra_files(['Rn067_A.hs'])], multimod_compile, ['rn067', '-v0'])

test('rn068', normal, compile, [''])

test('T17a', normal, compile, [''])
test('T17b', normal, compile, [''])
test('T17c', normal, compile, [''])
test('T17d', normal, compile, [''])
test('T17e', normal, compile, [''])

test('T1972', normal, compile, [''])
test('T2205', normal, compile, [''])

test('T2334', normal, compile, [''])
test('T2506', normal, compile, [''])
test('T2914', normal, compile, [''])
test('T3221', normal, compile, [''])
test('T3262', normal, compile, [''])
test('T3371', normal, compile, [''])
test('T1074b', normal, compile, ['-Wall'])
test('T3640', normal, compile, [''])
test('T3901', normal, compile, [''])
test('T1954', normal, compile, [''])
test('T3943', normal, compile, [''])

test('timing001', normal, compile, [''])
test('timing002', normal, compile, [''])
test('timing003', normal, compile, [''])

test('T1792_imports', [], run_command,
     ['$MAKE -s --no-print-directory T1792_imports'])

test('T3823', [], run_command, ['$MAKE -s --no-print-directory T3823'])

test('T4003', [], run_command, ['$MAKE -s --no-print-directory T4003'])
test('T1789', normal, compile, [''])
test('T1789_2', normal, compile, [''])
test('T3449', [], run_command, ['$MAKE -s --no-print-directory T3449'])
test('T4239', [], run_command, ['$MAKE -s --no-print-directory T4239'])
test('T4240', [], run_command, ['$MAKE -s --no-print-directory T4240'])

test('T4489', normal, compile, [''])
test('T4478', normal, compile, [''])
test('T4534', normal, compile, [''])

test('mc09', normal, compile, [''])
test('mc10', normal, compile, [''])
test('mc11', normal, compile, [''])
test('mc12', normal, compile, [''])
test('T2436', [], multimod_compile, ['T2436', '-v0'])
test('T5331', normal, compile, [''])   # Unused tyvar warnings
test('T5334', normal, compile, [''])   # Unused tyvar warnings
test('T5306', [], multimod_compile, ['T5306', '-v0'])
test('T5592', [], multimod_compile_and_run, ['T5592', '-v0'])
test('T5867', [], multimod_compile, ['T5867', '-v0'])
test('T6027', normal, compile, [''])
test('T6120', normal, compile, [''])
test('T7007', normal, compile, [''])
test('T7085', normal, compile, [''])
test('T7145a', normal, compile, ['-Wall -Werror'])
test('T7145b', normal, compile, ['-Wall'])
test('T6038', normal, compile, [''])
test('dodgy', [extra_files(['DodgyA.hs'])], multimod_compile, ['dodgy', '-v0'])
test('T7167', normal, compile, [''])
test('T7336', expect_broken(7336), compile, ['-Wall'])

test('T2435', normal, multimod_compile, ['T2435','-v0'])
test('T7672', normal, multimod_compile, ['T7672','-v0'])
test('T7963', [], run_command, ['$MAKE -s --no-print-directory T7963'])
test('T7969', [], run_command, ['$MAKE -s --no-print-directory T7969'])
test('T9127', normal, compile, [''])
test('T4426', normal, compile_fail, [''])
test('T9778', normal, compile, ['-fwarn-unticked-promoted-constructors'])
test('T10816', normal, compile, [''])
test('T11164', [], multimod_compile, ['T11164', '-v0'])
test('T11167', normal, compile, [''])
test('T11167_ambig', normal, compile, [''])
test('T10625', normal, compile, [''])
test('T11624', [], multimod_compile, ['T11624', ''])
test('T11662', [], multimod_compile, ['T11662', '-v0'])
test('T12127', [], multimod_compile, ['T12127', '-v0'])
test('T12533', normal, compile, [''])
test('T12597', normal, compile, [''])
test('T12548', normal, compile, [''])
test('T13132', normal, compile, [''])
test('T13646', normal, compile, [''])
test('LookupSub', [], multimod_compile, ['LookupSub', '-v0'])
