module Mod10 where

data T1 a = C1 a

data T2 a b = T2C1 a | T2C2 b

data T3 a b c = T3C1 a | T3C2 b | T3C3 c

data T4 a b c d = T4C1 a | T4C2 b | T4C3 c | T4C4 d

data T5 a b c d e = T5C1 a | T5C2 b | T5C3 c | T5C4 d | T5C5 e

data T6 a = T6C6 a

data T7 a b = T7C6 a | T7C7 b

data T8 a b c = T8C1 a | T8C2 b | T8C3 c

data T9 a b c d = T9C1 a | T9C2 b | T9C3 c | T9C4 d

data T10 a b c d e = T10C1 a | T10C2 b | T10C3 c | T10C4 d | T10C5 e
