#include <stdio.h>

void many_floats(float f1, float f2, float f3, float f4, float f5,
                 float f6, float f7, float f8, float f9, float f10,
                 float f11, float f12, float f13, float f14, double f15) {
  printf("%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n",
         f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, f15);
}
