test('completesig01', normal, compile, [''])
test('completesig02', normal, compile, [''])
test('Completesig03', normal, multimod_compile, ['Completesig03', '-Wall'])
test('completesig04', normal, compile_fail, [''])
test('completesig05', normal, compile, [''])
test('completesig06', normal, compile, [''])
test('completesig07', normal, compile, [''])
test('completesig08', normal, compile, [''])
test('completesig09', normal, compile, [''])
test('completesig10', normal, compile, [''])
test('completesig11', normal, compile, [''])
test('completesig12', normal, compile, [''])
test('completesig13', normal, compile, [''])
test('completesig14', normal, compile, [''])
test('completesig15', normal, compile_fail, [''])
test('T14253', expect_broken(14253), compile, [''])
