{-# LANGUAGE ConstraintKinds #-}

module T10451 where

type S a = ( Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a )

type T a = ( Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a
           , Eq a, Eq a, Eq a, Eq a)

