test('bkp01', normal, backpack_compile, ['-O'])
test('bkp02', normal, backpack_compile, [''])
test('bkp07', normal, backpack_compile, [''])
test('bkp08', normal, backpack_compile, [''])
test('bkp09', normal, backpack_compile, [''])
test('bkp10', normal, backpack_compile, [''])
test('bkp11', normal, backpack_compile, [''])
test('bkp12', normal, backpack_compile, [''])
test('bkp14', normal, backpack_compile, [''])
test('bkp15', normal, backpack_compile, [''])
test('bkp16', normal, backpack_compile, [''])
test('bkp17', normal, backpack_compile, [''])
test('bkp18', normal, backpack_compile, [''])
test('bkp19', normal, backpack_compile, [''])
test('bkp20', normal, backpack_compile, [''])
test('bkp21', normal, backpack_compile, [''])
test('bkp23', normal, backpack_compile, [''])
test('bkp24', normal, backpack_compile, [''])
test('bkp25', normal, backpack_compile, [''])
test('bkp26', normal, backpack_compile, [''])
test('bkp27', normal, backpack_compile, [''])
test('bkp28', normal, backpack_compile, [''])
test('bkp29', normal, backpack_compile, [''])
test('bkp30', normal, backpack_compile, [''])
test('bkp31', normal, backpack_compile, [''])
test('bkp32', normal, backpack_compile, [''])
test('bkp33', normal, backpack_compile, [''])
test('bkp34', normal, backpack_compile, [''])
test('bkp35', normal, backpack_compile, [''])
test('bkp36', normal, backpack_compile, [''])
test('bkp37', normal, backpack_compile, [''])
test('bkp38', normal, backpack_compile, [''])
test('bkp39', normal, backpack_compile, [''])
test('bkp40', normal, backpack_compile, [''])
test('bkp41', normal, backpack_compile, [''])
test('bkp42', normal, backpack_compile, [''])
test('bkp43', normal, backpack_compile, [''])
test('bkp44', normal, backpack_compile, [''])
test('bkp45', normal, backpack_compile, [''])
test('bkp46', normal, backpack_compile, [''])
test('bkp47', normal, backpack_compile, [''])
test('bkp48', normal, backpack_compile, [''])
test('bkp49', normal, backpack_compile, [''])
test('bkp50', normal, backpack_compile, [''])
test('bkp51', normal, backpack_compile, [''])
test('bkp52', normal, backpack_compile, [''])
test('bkp53', normal, backpack_compile, [''])
test('bkp54', normal, backpack_compile, [''])
test('bkp55', normal, backpack_compile, [''])

test('T13140', normal, backpack_compile, [''])
test('T13149', expect_broken(13149), backpack_compile, [''])
test('T13214', normal, backpack_compile, [''])
test('T13250', normal, backpack_compile, [''])
test('T13323', normal, backpack_compile, [''])
